indexing

	description:

		"Reader which connects to a source."

	library: "Gobo Eiffel Pipework Library"
	copyright: "Copyright (c) 2001, Simon Parker and others"
	license: "Eiffel Forum License v1 (see forum.txt)"
	date: "$Date: $"
	revision: "$Revision: $"


deferred class

	PW_SOURCE_READER [OUTWARD]

inherit

	PW_READER [OUTWARD]
		redefine
			close,
			is_closable
		end

feature -- Initialization

	make is
			-- Prepare an unconnected reader.
		do
			if is_closable then
				close
			end
		ensure then
			closed: not is_open_read
		end

feature -- Status

	is_closable: BOOLEAN is
		do
			Result := is_connected
		end

	is_connected: BOOLEAN is
			-- Has a source been established?
		do
			Result := source /= Void
		end

	is_open_read: BOOLEAN is
		do
			Result := is_connected
		end


feature -- Basic operations

	close is
			-- Try to close input stream if it is closable. Set
			-- `is_open_read' to false if operation was successful.
		do
			disconnect
		end


feature -- Connection management

	source: ANY
			-- Where raw material comes from.
			-- Descendents may redefine this to establish type.

	connect (new_source: like source) is
			-- Use new_source until further notice. Postcondition should
			-- satisfy `read' precondition.
		require
			unconnected: not is_connected
			new_source_not_void: new_source /= Void
		do
			source := new_source
		ensure
			connected: is_connected
		end

	disconnect is
			-- Stop using `source'.
		do
			source := Void
		ensure
			source_void: source = Void
		end

	reconnect (new_source: like source) is
			-- Replace `source' with `new_source'. Postcondition should
			-- satisfy `read' precondition.
		require
			open: is_open_read
			new_source_not_void: new_source /= Void
		do
			disconnect
			reconnect (new_source)
		ensure
			connected: is_connected
		end


invariant

	source_present_if_open_read: is_open_read implies source /= Void

end
